' ========================================================================================
' GdipPathIterRewind
' ========================================================================================

' SED_PBWIN
#COMPILE EXE
#DIM ALL
#INCLUDE "GDIPLUS.INC"

' ========================================================================================
' The following example creates a GraphicsPath object and adds five figures to the path.
' The code passes the address of that GraphicsPath object to the GdipCreatePathIter
' function to create an iterator that is associated with the path. The code calls the
' GdipPathIterNextSubpathPath function twice to retrieve the second figure in the path.
' The GdipDrawPath function draws that path in blue. Next, the code calls the
' GdipPathIterRewind function and then calls GdipPathIterNextSubpathPath once to obtain
' the first figure in the path. The GdipDrawPath function draws that figure in red.
' ========================================================================================
SUB GDIP_PathIterRewind (BYVAL hdc AS DWORD)

   LOCAL hStatus AS LONG
   LOCAL pGraphics AS DWORD
   LOCAL pPath AS DWORD
   LOCAL pIterator AS DWORD
   LOCAL pSubPath AS DWORD
   LOCAL pBluePen AS DWORD
   LOCAL pRedPen AS DWORD
   LOCAL count AS LONG
   LOCAL isClosed AS LONG

   hStatus = GdipCreateFromHDC(hdc, pGraphics)

   ' // Create a graphics path with five figures (subpaths).
   hStatus = GdipCreatePath(%FillModeAlternate, pPath)

   hStatus = GdipAddPathRectangle(pPath, 20, 20, 60, 30)       ' // Subpath count is 1.

   hStatus = GdipAddPathLine(pPath, 100, 20, 160, 50)          ' // Subpath count is 2.
   hStatus = GdipAddPathArc(pPath, 180, 20, 60, 30, 0, 180)

   hStatus = GdipAddPathRectangle(pPath, 260, 20, 60, 30)      ' // Subpath count is 3.

   hStatus = GdipAddPathLine(pPath, 340, 20, 400, 50)          ' // Subpath count is 4.
   hStatus = GdipAddPathArc(pPath, 340, 20, 60, 30, 0, 180)
   hStatus = GdipClosePathFigure(pPath)

   hStatus = GdipAddPathRectangle(pPath, 420, 20, 60, 30)      ' // Subpath count is 5.

   ' // Create an iterator, and associate it with the path.
   hStatus = GdipCreatePathIter(pIterator, pPath)

   ' // Get the second subpath by calling NextSubpath twice.
   hStatus = GdipCreatePath(%FillModeAlternate, pSubPath)
   hStatus = GdipPathIterNextSubpathPath(pIterator, count, pSubPath, isClosed)
   hStatus = GdipPathIterNextSubpathPath(pIterator, count, pSubPath, isClosed)

   ' // Draw the second figure in blue.
   hStatus = GdipCreatePen1(GDIP_ARGB(255, 0, 0, 255), 1, %UnitWorld, pBluePen)
   hStatus = GdipDrawPath(pGraphics, pBluePen, pSubPath)

   ' // Rewind the iterator, and get the first figure in the path.
   hStatus = GdipPathIterRewind(pIterator)
   hStatus = GdipPathIterNextSubpathPath(pIterator, count, pSubPath, isClosed)

   ' // Draw the first figure in red.
   hStatus = GdipCreatePen1(GDIP_ARGB(255, 255, 0, 0), 1, %UnitWorld, pRedPen)
   hStatus = GdipDrawPath(pGraphics, pRedPen, pSubPath)

   ' // Cleanup
   IF pBluePen THEN GdipDeletePen(pBluePen)
   IF pRedPen THEN GdipDeletePen(pRedPen)
   IF pIterator THEN GdipDeletePathIter(pIterator)
   IF pSubPath THEN GdipDeletePath(pSubPath)
   IF pPath THEN GdipDeletePath(pPath)
   IF pGraphics THEN GdipDeleteGraphics(pGraphics)

END SUB
' ========================================================================================

' ========================================================================================
' Main
' ========================================================================================
FUNCTION WINMAIN (BYVAL hInstance AS DWORD, BYVAL hPrevInstance AS DWORD, BYVAL lpszCmdLine AS ASCIIZ PTR, BYVAL nCmdShow AS LONG) AS LONG

   LOCAL hr AS LONG
   LOCAL hDlg AS DWORD
   LOCAL token AS DWORD
   LOCAL StartupInput AS GdiplusStartupInput

   ' Initialize GDI+
   StartupInput.GdiplusVersion = 1
   hr = GdiplusStartup(token, StartupInput, BYVAL %NULL)
   IF hr THEN
      MSGBOX "Error initializing GDI+"
      EXIT FUNCTION
   END IF

   DIALOG NEW 0, "GdipPathIterRewind", , , 280, 170, %WS_OVERLAPPED OR %WS_THICKFRAME OR %WS_SYSMENU OR _
   %WS_MINIMIZEBOX OR %WS_MAXIMIZEBOX OR %WS_VISIBLE OR %DS_CENTER TO hDlg
   DIALOG SET COLOR hDlg, %BLACK, %WHITE
   DIALOG SHOW MODAL hDlg, CALL DlgProc

   ' Shutdown GDI+
   GdiplusShutdown token

END FUNCTION
' ========================================================================================

' ========================================================================================
' Main Dialog procedure
' ========================================================================================
CALLBACK FUNCTION DlgProc() AS LONG

   LOCAL hDC AS DWORD
   LOCAL ps AS PAINTSTRUCT

   SELECT CASE CBMSG

      CASE %WM_COMMAND
         SELECT CASE CBCTL
            CASE %IDCANCEL
               IF CBCTLMSG = %BN_CLICKED THEN DIALOG END CBHNDL, 0
         END SELECT

      CASE %WM_PAINT
         hDC = BeginPaint(CBHNDL, ps)
         GDIP_PathIterRewind hDC
         EndPaint(CBHNDL, ps)

   END SELECT

END FUNCTION
' ========================================================================================
